# Test code that prevent Ctrl + C
# against a little mess with files,
# for example when manifest.json replaced to minified version for some browser
# https://unix.stackexchange.com/a/48432/34318
trap "exit" INT

# if current directory does not contains build.sh - message and exit
if ! [[ $(ls) = *build.sh* ]]; then
    echo 'ERROR: run this build script from the directory where it is live'
    exit
fi

# On Gentoo do not use -g (--global) against conflicts with Portage
DEPENDENCIES='For installing all dependencies: npm install -g html-minifier stylelint postcss-cli cssnano eslint terser minify-json'

# IF syntax from https://stackoverflow.com/a/26759734/1879101
if ! [[ -x "$(command -v node_modules/.bin/html-minifier)" ]]; then
    echo "Please install html-minifier: 'npm install html-minifier'"
    echo $DEPENDENCIES
    exit
fi

# $? is a return code: 0 for success, 1 for error
hash node_modules/.bin/stylelint
if [ $? -ne 0 ]; then
    echo 'Please install stylelint: `npm install stylelint`'
    echo $DEPENDENCIES
    exit
fi

node_modules/.bin/stylelint *.css
if [ $? -ne 0 ]; then
    echo 'Fix CSS errors before build'
    echo $DEPENDENCIES
    exit
fi

hash node_modules/.bin/postcss 2> /dev/null
if [ $? -ne 0 ]; then
    echo "Please install postcss: 'npm install postcss-cli'"
    echo $DEPENDENCIES
    exit
fi

node_modules/.bin/postcss snackbar.css --use cssnano --no-map > /dev/null
if [ $? -ne 0 ]; then
    echo "Please install cssnano: 'npm install cssnano'"
    echo $DEPENDENCIES
    exit
fi

hash node_modules/.bin/eslint 2> /dev/null
if [ $? -ne 0 ]; then
    echo 'Please install eslint: `npm install eslint`'
    echo $DEPENDENCIES
    exit
fi

node_modules/.bin/eslint .
if [ $? -ne 0 ]; then

    echo Eslint version: "$(node_modules/.bin/eslint --version)", fix eslint errors before build.
    exit
fi

if ! [[ "$(node_modules/.bin/terser --version)" =~ ^terser ]]; then
    echo "Please install terser: 'npm install terser'."
    echo "Their repo (just for your information): https://github.com/terser-js/terser"
    echo $DEPENDENCIES
    exit
fi

hash node_modules/.bin/minify-json 2> /dev/null  # /dev/null: do not print result
if [ $? -ne 0 ]; then
    echo 'Please install minify-json: `npm install minify-json`';
    echo $DEPENDENCIES
    exit
fi


# Without .git directory only archive for reviewers - checksum must be the same
ls ../.git > /dev/null
if [ $? -ne 0 ]; then
    set -- 'prod'
    echo 'Building PROD version'
    # This is like $@ = 'prod'
else
    STATUS=$(git status .)
    if [[ $STATUS = *'Changes not staged for commit'* || $STATUS = *'untracked files present'* ]] && [[ $@ == 'prod' ]]; then
        echo 'Commit everything in extension folder before build: git commit . -p'
        echo 'Reasons:'
        echo '1) For correct archive for reviewers - because it is build by git from the last commit'
        echo '2) For git tag on correct commit'
        exit
    fi
fi


mkdir public
cp -r manifest.json _locales public/

find images/ \( -name '*.png' -o -name '*svg' \) -exec rsync --relative {} public \;
cd public

find . -type f -name "messages.json" | xargs perl -pi -e 's/",/"/g'
find . -type f -name "messages.json" | xargs perl -pi -e 's/"description".+//g'

../node_modules/.bin/minify-json .
if [ $? -ne 0 ]; then
    echo 'Fix JSONs in _locales/, build stopped'
    cd -
    rm -rf public 2> /dev/null
    exit
fi
cd -

mkdir build 2> /dev/null
PWD=`pwd`
CHROMIUM_FILEPATH_TEST=$PWD'/build/chromium.test.zip'
FIREFOX_FILEPATH_TEST=$PWD'/build/firefox.test.zip'

CHROMIUM_FILEPATH_PROD=$PWD'/build/chromium.prod.zip'
FIREFOX_FILEPATH_PROD=$PWD'/build/firefox.prod.zip'
# Prod and test archives for case when I will need to add hotfix without using git branches and stash

REVIEWABLE_FILEPATH=$PWD'/build/intelligent-speaker.source-for-stores-review.zip'

VERSION=`date +%Y.%-m.%-d.%-H%M`

perl -p -i -e 's/.+common.+//g' eventPage.html popup.html
perl -p -i -e 's/.+downloadAudio.+//g' eventPage.html
perl -p -i -e 's/data-preview=".+"//g' popup.html
node_modules/.bin/html-minifier --collapse-whitespace --remove-comments popup.html > public/popup.html
node_modules/.bin/html-minifier --collapse-whitespace --remove-comments eventPage.html > public/eventPage.html
git checkout popup.html eventPage.html  # Restore <script src='common.js'></script>

for filename in *.css; do
    node_modules/.bin/postcss $filename --use cssnano --verbose --no-map > public/$filename
done

node_modules/.bin/terser eventPage.js common.js downloadAudio.js --compress drop_console=true --mangle --toplevel --verbose --source-map --output public/eventPage.js
node_modules/.bin/terser popup.js common.js --compress drop_console=true --mangle --toplevel --verbose --source-map --output public/popup.js
node_modules/.bin/terser contentScript.js --compress drop_console=true --mangle --toplevel --verbose --source-map --output public/contentScript.js

# Alter manifest for Firefox first because it is simpler to restore 'incognito' than 'application'
python3 -c "
import json
import sys

with open('manifest.json') as file:
    json_manifest = json.load(file)

# https://developer.chrome.com/extensions/manifest/version
if json_manifest['version'] == '0':  # When build developer, Mozilla review must build to the same
    json_manifest['version'] = sys.argv[1]

if sys.argv[2] and sys.argv[2].lower() == 'prod':
    with open ('public/manifest.versioned.json', 'w') as file:
        file.write(json.dumps(json_manifest, indent=2))

# Firefox still does not support it; set to true against warning to user in addons screen
del json_manifest['background']['persistent']

del json_manifest['minimum_chrome_version']

with open('public/manifest.json', 'w') as file:
    file.write(json.dumps(json_manifest))
" $VERSION $@

cd public

../node_modules/.bin/minify-json manifest.json  # Python saved in one line but with spaces

if [[ $@ == 'prod' ]]; then
    rm $FIREFOX_FILEPATH_PROD 2> /dev/null  # remove because maybe some file was deleted from archive
    zip -r $FIREFOX_FILEPATH_PROD . -x manifest.versioned.json *.js.map
else
    rm $FIREFOX_FILEPATH_TEST 2> /dev/null
    zip -r $FIREFOX_FILEPATH_TEST . -x *.js.map
fi

python3 -c "
import json
import sys

with open('manifest.json') as file:
    json_manifest = json.load(file)
    json_manifest.pop('applications')

    json_manifest['incognito'] = 'not_allowed'
    json_manifest['background']['persistent'] = False
    json_manifest['container'] = 'GOOGLE_DRIVE'
    json_manifest['offline_enabled'] = True
    json_manifest['minimum_chrome_version'] = '60'

    # TODO delete after resolving this https://bugzilla.mozilla.org/show_bug.cgi?id=1432083
    json_manifest['permissions'].pop()

    # Quote from https://docs.python.org/3/library/sys.html:
    # The list of command line arguments passed to a Python script.
    # argv[0] is the script name (it is operating system dependent whether this is a full pathname or not).
    # If the command was executed using the -c command line option to the interpreter,
    # argv[0] is set to the string '-c'.
    # If no script name was passed to the Python interpreter, argv[0] is the empty string.
    if len(sys.argv) == 1:
        json_manifest['name'] = 'Intelligent Speaker [QA]'
    elif sys.argv[1].lower() == 'uat':
        json_manifest['name'] = 'Intelligent Speaker [UAT]'

with open('manifest.json', 'w') as file:
    file.write(json.dumps(json_manifest))
" $@

../node_modules/.bin/minify-json manifest.json

if [[ $@ == 'prod' ]]; then
    rm $CHROMIUM_FILEPATH_PROD 2> /dev/null
    zip -r $CHROMIUM_FILEPATH_PROD . -x manifest.versioned.json *.js.map

    cd ..
    git archive --output $REVIEWABLE_FILEPATH HEAD -9
    cd -

    # Check if build by developer - not by reviewer
    if [[ $(git status > /dev/null) -eq 0 ]]; then
        mv manifest.versioned.json manifest.json
        zip $REVIEWABLE_FILEPATH manifest.json *.js.map
    fi

    # For reviewers at Mozilla and Opera
    echo ''
    echo 'Build finished, resulting deployment packages available at:'
    echo $CHROMIUM_FILEPATH_PROD
    echo $FIREFOX_FILEPATH_PROD
else
    rm $CHROMIUM_FILEPATH_TEST 2> /dev/null
    zip -r $CHROMIUM_FILEPATH_TEST . -x *.js.map
fi

cd ..
DIRECTORY=/tmp/intelligent-speaker--extension/
mkdir "${DIRECTORY}" 2> /dev/null # If already exists
mv public "${DIRECTORY}${VERSION}"

echo 'Content of deployment package for Chromium:'
if [[ $@ == 'prod' ]]; then
    unzip -l $CHROMIUM_FILEPATH_PROD
else
    unzip -l $CHROMIUM_FILEPATH_TEST
fi

echo
echo 'Filesize of deployment package for Chromium:'
if [[ $@ == 'prod' ]]; then
    ls -lh $CHROMIUM_FILEPATH_PROD
else
    ls -lh $CHROMIUM_FILEPATH_TEST
fi
