function downloadLocalOGGFromIndexedDB(id) {  // eslint-disable-line no-unused-vars

    /**
     * substring() and replace() against 'Invalid filename' JS error
     */
    const _buildFilename = title => {
        return title
            .substring(0, 90)
            .replace(/[:|*?\\"]/g, '-') + '.ogg';
    };

    /**
     * @see {@link https://developer.chrome.com/extensions/downloads|documentation}
     */
    const _download = (url, title) => {
        chrome.downloads.download({
            url,
            'filename': _buildFilename(title),
            'saveAs': true
        }, downloadId => {
            if (!downloadId && chrome.runtime.lastError.message === 'Invalid filename') {
                // For example when filename is 'मिन्‍स्‍क - विकिपीडिया'
                _download(url, 'intelligent-speaker');
            }
        });
    };
    const callback = _ => {
        // Callback because chrome.downloads does not exists before allowed permission
        getFromObjectStore(+id).then(async feedEntity => {  // eslint-disable-line no-undef
            const blob = feedEntity['cache'].length == 1 ? feedEntity['cache'][0] : new Blob(
                await _randomizeSerials(feedEntity['cache']),
                {'type': 'audio/ogg'}
            );
            const url = URL.createObjectURL(blob);
            _download(url, feedEntity['title']);
        });
    };

    const permission = {
        'permissions': ['downloads']
    };

    /* Browser will check automatically, but because of defect on Firefox
     * I altered build.sh that inject this permission. See
     * https://bugzilla.mozilla.org/show_bug.cgi?id=1432083
     */
    chrome.permissions.contains(permission, isAllowed => {
        if (isAllowed) {
            callback();
        } else {
            chrome.permissions.request(permission, _ => callback());
        }
    });

}
/* Without that not all players can play, see
 * https://stackoverflow.com/questions/53290693
 */
async function _randomizeSerials(blobs) {
    return await Promise.all(blobs.map(_randomizeSerial));
}
async function _randomizeSerial(blob, i) {
    const typedArray = new Uint8Array(await new Response(blob).arrayBuffer());

    return new Blob(
        _getPages(typedArray).map(oggPage => _setSerialForPage(oggPage, i))
    );
}
function _getPages(typedArray) {
    const pages = [];
    const O = 79;   // 4F
    const g = 103;  // 67
    const S = 83;   // 53
    let iPrevious = 0;
    typedArray.forEach((val, i) => {
        if (i &&
            val               === O &&
            typedArray[i + 1] === g &&
            typedArray[i + 2] === g &&
            typedArray[i + 3] === S) {
            pages.push(typedArray.slice(iPrevious, i));
            iPrevious = i;
        }
    });

    pages.push(typedArray.slice(iPrevious));

    return pages;
}
function _makeCRC32Table() {
    // From https://stackoverflow.com/questions/53438815/hot-to-build-crc32-table-for-ogg
    const polynomial = 79764919;
    const mask = 2147483648;
    const CRCTable = new Uint32Array(256);
    for (let i = 256; i--;) {
        let char = i << 24;
        for (let j = 8; j--;) {
            char = char & mask ? polynomial ^ char << 1 : char << 1;
        }
        CRCTable[i] = char;
    }
    return CRCTable;
}
function _setSerialForPage(oggPage, i) {
    if (oggPage[17] === 0 && oggPage[16] === 0 && oggPage[15] === 0 && oggPage[14] === 0) {
        console.log('Serials is zeroes - set them for correct concatenation');
        // AWS Polly returns Oggs with zeroed serials

        /* See Ogg page header documentation at
         * https://xiph.org/vorbis/doc/framing.html
         */
        const max = 255;  // Value in Uint8Array
        oggPage[17] = 1;  // Do not remain 00000000 against warning from ogginfo
        oggPage[15] = i <= max ? 0 : i / max;
        oggPage[14] = i <= max ? i : max;

        _zeroesExistingCRC32(oggPage);
        const CRCTable = window.CRCTable || (window.CRCTable = _makeCRC32Table());
        const crc = oggPage.reduce(
            (crc, byte) => crc << 8 >>> 0 ^ CRCTable[crc >>> 24 ^ byte], 0
        ) >>> 0;

        const hex = crc.toString(16).padStart(8, '0');
        oggPage[25] = parseInt(hex.substring(0, 2), 16);
        oggPage[24] = parseInt(hex.substring(2, 4), 16);
        oggPage[23] = parseInt(hex.substring(4, 6), 16);
        oggPage[22] = parseInt(hex.substring(6), 16);

        return oggPage;

    } else {
        // Wav from Microsoft converted on our tts.go lambda with oggenc
        console.log('Serials already not zeroes');
        return oggPage;
    }
}
function _zeroesExistingCRC32(oggPage) {
    oggPage[22] = 0;
    oggPage[23] = 0;
    oggPage[24] = 0;
    oggPage[25] = 0;
}
